#cython: language_level=3

from obitools3.utils cimport bytes2str, str2bytes, tostr, tobytes

from .capi.obiview cimport NUC_SEQUENCE_COLUMN, \
                           ID_COLUMN, \
                           DEFINITION_COLUMN, \
                           QUALITY_COLUMN, \
                           COUNT_COLUMN


cdef class Seq(dict) :
    def __init__(self, object id, object seq, object definition=None, object quality=None, int offset=-1, object tags=None) :
        cdef object k 
        cdef bytes k_b
        self[ID_COLUMN] = tobytes(id)
        self.seq = tobytes(seq)
        if definition is not None :
            self.definition = tobytes(definition)
        self._annotations = {}
        if tags is not None :
            for k in tags:
                k_b = tobytes(k)
                self[k_b] = tags[k_b]


    def __contains__(self, object key):
        return dict.__contains__(self, tobytes(key))
            
        
    # sequence id property getter and setter
    @property
    def id(self):  # @ReservedAssignment
        return self[ID_COLUMN]
    
    @id.setter
    def id(self, object new_id):  # @ReservedAssignment @DuplicatedSignature
        self[ID_COLUMN] = tobytes(new_id)

    # sequence annotations property getter and setter
    @property
    def annotations(self):  # @ReservedAssignment
        return self._annotations
    
    @annotations.setter
    def annotations(self, object annotations):  # @ReservedAssignment @DuplicatedSignature
        self._annotations = annotations
        
    # sequence property getter and setter
    @property
    def seq(self):
        return self[b"SEQ"]
    
    @seq.setter
    def seq(self, object new_seq):  # @DuplicatedSignature
        self[b"SEQ"] = tobytes(new_seq)
        
    # sequence definition property getter and setter
    @property
    def definition(self):
        return self[DEFINITION_COLUMN]
    
    @definition.setter
    def definition(self, object new_definition):  # @DuplicatedSignature
        self[DEFINITION_COLUMN] = tobytes(new_definition)
        
        
cdef class Nuc_Seq(Seq) :
    
    # nuc sequence property getter and setter
    @property
    def seq(self):
        return self._seq
    
    @seq.setter
    def seq(self, object new_seq):  # @DuplicatedSignature
        self._seq = new_seq
        self[NUC_SEQUENCE_COLUMN] = tobytes(new_seq)
        
    # sequence quality property getter and setter
    @property
    def quality(self):
        return self[QUALITY_COLUMN]
    
    @quality.setter
    def quality(self, object new_quality):  # @DuplicatedSignature
        self[QUALITY_COLUMN] = tobytes(new_quality)
        
#    cpdef str reverse_complement(self) :    TODO in C ?
#        pass


cdef class Seq_Stored(Line) :
    
    # sequence id property getter and setter
    @property
    def id(self):               # @ReservedAssignment @DuplicatedSignature
        return self._view.get_column(ID_COLUMN).get_line(self.index)
    
    @id.setter
    def id(self, object new_id):   # @ReservedAssignment @DuplicatedSignature
        self._view.get_column(ID_COLUMN).set_line(self._index, tobytes(new_id))
        
    # sequence definition property getter and setter
    @property
    def definition(self):
        return self._view.get_column(DEFINITION_COLUMN).get_line(self.index)
    
    @definition.setter
    def definition(self, object new_def): # @DuplicatedSignature
        self._view.get_column(DEFINITION_COLUMN).set_line(self._index, tobytes(new_def))


cdef class Nuc_Seq_Stored(Seq_Stored) :
    
    cpdef set_quality_int(self, list new_qual):
        self._view.get_column(QUALITY_COLUMN).set_line(self.index, new_qual)

    cpdef set_quality_char(self, bytes new_qual, int offset=-1):
        self._view.get_column(QUALITY_COLUMN).set_str_line(self.index, new_qual, offset=offset)
    
    # nuc_seq property getter and setter
    @property
    def nuc_seq(self):
        return self._view.get_column(NUC_SEQUENCE_COLUMN).get_line(self.index)
    
    @nuc_seq.setter
    def nuc_seq(self, object new_seq): # @DuplicatedSignature
        self._view.get_column(NUC_SEQUENCE_COLUMN).set_line(self.index, tobytes(new_seq))
        
    # quality property getter and setter
    @property
    def quality(self):
        return self._view.get_column(QUALITY_COLUMN).get_line(self.index)
    
    @quality.setter
    def quality(self, object new_qual): # @DuplicatedSignature
        if (new_qual is None) or (type(new_qual) == list) : # TODO check that quality column exists
            self._view.get_column(QUALITY_COLUMN).set_line(self._index, new_qual)
        elif (type(new_qual) == str) or (type(new_qual) == bytes)  :  # Quality is in str form
            self._view.get_column(QUALITY_COLUMN).set_str_line(self._index, tobytes(new_qual))
        else :
            raise Exception("Sequence quality in unrecognized format")
            
    # quality character string property getter
    @property
    def quality_str(self):
        return self._view.get_column(QUALITY_COLUMN).get_str_line(self._index)
    
    
    def __str__(self):
        return bytes2str(self.nuc_seq)

    
#    cpdef str reverse_complement(self) :    TODO in C ?
#        pass

    # TODO static method to import OBI_Nuc_Seq to OBI_Nuc_Seq_Stored ?
    
    